#include <Windows.h>
#include <shlwapi.h>
#include <iostream>
#include "lua.hpp"
#include <luabind/luabind.hpp>

using namespace std;
using namespace luabind;

#pragma comment(lib, "shlwapi.lib")


class CLuaClassTest
{
private:
    static CLuaClassTest* m_pInstance;
public:
    CLuaClassTest()
    {
        cout << "from ctor" << endl;
    }

    void valid_member()
    {
        cout << "Greeting!" << endl;
    }

    static CLuaClassTest* QueryInstance()
    {
        if (!m_pInstance)
        {
            m_pInstance = new CLuaClassTest;
        }

        return m_pInstance;
    }
};

CLuaClassTest* CLuaClassTest::m_pInstance = NULL;

void g_func()
{
    cout << "from g_func" << endl;
}

int main()
{
    cout << "Hello, lua_pcall test!" << endl;

    lua_State* L = lua_open();
    lua_gc(L, LUA_GCSTOP, 0);  
    luaL_openlibs(L);
    luabind::open(L);

    module(L, "TestModule")    
        [
            class_<CLuaClassTest>("CLuaClassTest")
            //.def( constructor<>())
            .def("valid_member", &CLuaClassTest::valid_member)
            .scope
            [
                def("QueryInstance", &CLuaClassTest::QueryInstance)
            ],

            def("g_func", &g_func)
        ];

    lua_gc(L, LUA_GCRESTART, 0);
    char szLuaScript[MAX_PATH] = {0};
    GetModuleFileNameA(NULL, szLuaScript, MAX_PATH);
    PathRemoveFileSpecA(szLuaScript);
    strcat_s(szLuaScript, "\\luaClassTest.lua");
    
    int nerror = luaL_dofile(L, szLuaScript);
    if (nerror) 
    {
        cout << lua_tostring(L, -1) << endl; 
        lua_pop(L, 1);       
    }
    else
    {
        //lua_getglobal(L, "crash_test");
        //if (lua_pcall(L, 0, 0, 0) != 0)
        //{
        //    cout <<  lua_tostring(L, -1) << endl;
        //    lua_pop(L, 1);
        //}

        //lua_getglobal(L, "invalid_static_member");
        //if (lua_pcall(L, 0, 0, 0) != 0)
        //{
        //    cout <<  lua_tostring(L, -1) << endl;
        //    lua_pop(L, 1);
        //}

        //lua_getglobal(L, "nil_value_call");
        //if (lua_pcall(L, 0, 0, 0) != 0)
        //{
        //    cout <<  lua_tostring(L, -1) << endl;
        //    lua_pop(L, 1);
        //}

        try
        {
            call_function<void>(L, "crash_test");            
        }
        catch(error& e)
        {
            object error_msg(from_stack(e.state(), -1));
            cerr << error_msg;
            lua_pop(L, 1);
        }
        try
        {
            call_function<void>(L, "invalid_static_member");            
        }
        catch(error& e)
        {
            object error_msg(from_stack(e.state(), -1));
            cerr << error_msg;
            lua_pop(L, 1);
        }

        try
        {
            call_function<void>(L, "nil_value_call");            
        }
        catch(error& e)
        {
            object error_msg(from_stack(e.state(), -1));
            cerr << error_msg;
            lua_pop(L, 1);
        }

    }

    lua_close(L);
    return 0;
}